#include "MyAnalysis.hh"
#include "G4RunManager.hh" 
#include "TFile.h"
#include "TH1F.h"
#include "TH2F.h"
#include "TH3F.h"
#include "TNtuple.h"


MyAnalysis* MyAnalysis::instance = 0;

 
MyAnalysis::MyAnalysis() {

  // Declare and open here the histograms and/or ntuples.
//  histo_dedx = new TH1F( "dE/dx (GeV/(Nevt*cm))", "dE/dx", 125,-300,300);
//  histo_xpos  = new TH1F( "x (cm)","Hit positions",125,-300,300);
//  histo_edep  = new TH1F( "E (GeV/Nevt) ","Energy deposits",100,0,100);
//  histo_edep_vs_org_en = new TH2F( "Edep","Edep",30,0,800,100,0,100);
 // histo_pos = new TH3F("Position","Hit position",100,-100,100,100,-100,100,100,-100,100);
  nt_hits = new TNtuple("Hits","Hits","x:y:z:event:rhad:charge:edep:e");
histo_edep_cross = new TH2F("edep_cross","E (MeV / cm^3)",100,-50,50,100,-50,50);
histo_edep_trans = new TH2F("edep_trans","E vs depth",100,-500,+500,50,0,50);
histo_edep_trans2 = new TH1F("edep_trans2","E (MeV /cm^3)",50,0,50);   
  Nevt=0;
}


MyAnalysis::~MyAnalysis() {
  
  G4cout<<"Writing histograms to file..."<<G4endl;
  TFile histofile("myfile.root","RECREATE","My test Root file");
 // RenormToNevt(histo_dedx);
 // RenormToBinWidth(histo_dedx);
 // histo_dedx->Write();

 // RenormToNevt(histo_edep);
 // histo_edep->Write();

 // histo_xpos->Write();

 // histo_edep_vs_org_en->Write();

 // histo_pos->Write();
 histo_edep_cross->Scale(1./Nevt);
 histo_edep_trans->Scale(1./Nevt); 
 RenormToNevt(histo_edep_trans2);
 RenormToRadius(histo_edep_trans2);
 histo_edep_trans->Write();
 histo_edep_cross->Write();
 histo_edep_trans2->Write();

 nt_hits->Write();
  // Close here the histograms and/or ntuples.

}


MyAnalysis* MyAnalysis::getInstance() {
  if ( instance == 0 ) instance = new MyAnalysis();
  return instance;
}

//void MyAnalysis::fill(const G4int numHits, const G4double energyDeposited, const G4double distance) {
void MyAnalysis::fill(const MyTrackerHitsCollection* MyHits) {

  //This is perhaps a bad way of counting events.
  Nevt++;

  //  G4cout<<"Filling histos for this event..."<<G4endl;

  G4int numTrackerHits = MyHits->entries();
  G4cout<<"Number of tracker hits: "<<numTrackerHits<<G4endl;
  for ( G4int i=0; i < numTrackerHits; i++ ) {
  int rhad=0;
    if ( (*MyHits)[i]->GetParticleDefinition()->GetParticleType()=="rhadron" )// Only looking at R-hadron
      {
        rhad=1;
	G4double xpos = (*MyHits)[i]->GetPosition().x();
	G4double e = (*MyHits)[i]->Get4Mom().e();
	G4double edep = (*MyHits)[i]->GetEdep();
	//    G4cout<<"x = "<<xpos/cm<<" cm, E = "<<edep/MeV<<" MeV"<<G4endl;
//	histo_dedx->Fill(xpos/cm,edep/MeV);
//	histo_xpos->Fill(xpos/cm,1.);
//	histo_edep->Fill(edep/MeV);
//	histo_edep_vs_org_en->Fill(edep/GeV,e/GeV);
    
}
    double x=(*MyHits)[i]->GetPosition().x();
    double y=(*MyHits)[i]->GetPosition().y();
    double z=(*MyHits)[i]->GetPosition().z();
    double edep=(*MyHits)[i]->GetEdep()/GeV;

    histo_edep_cross->Fill(z/cm,y/cm,edep/GeV);   
    histo_edep_trans->Fill(x/cm,sqrt(z*z+y*y)/cm,edep/GeV);
    histo_edep_trans2->Fill(sqrt(z*z+y*y)/cm,edep/GeV);
    
  //    histo_pos->Fill((*MyHits)[i]->GetPosition().x(),(*MyHits)[i]->GetPosition().y(),(*MyHits)[i]->GetPosition().z());
//    nt_hits->Fill((*MyHits)[i]->GetPosition().x()/mm,(*MyHits)[i]->GetPosition().y()/mm,(*MyHits)[i]->GetPosition().z()/mm,Nevt,rhad,(*MyHits)[i]->GetParticleDefinition()->GetPDGCharge(),(*MyHits)[i]->GetEdep()/GeV,(*MyHits)[i]->Get4Mom().e()/GeV);
    
    
    /*
    if((*MyHits)[i]->GetParticleDefinition()->GetParticleType()=="rhadron")
      G4cout <<"Hit was: "<<(*MyHits)[i]->GetParticleDefinition()->GetParticleName()<<G4endl;
    */
  }

}

void MyAnalysis::RenormToNevt(TH1F* histo)
{
  for (G4int i=1; i<=histo->GetNbinsX(); i++){
    double cont = histo->GetBinContent(i); 
    histo->SetBinContent(i,cont/Nevt);
  }
}

void MyAnalysis::RenormToBinWidth(TH1F* histo)
{
  for (G4int i=1; i<=histo->GetNbinsX(); i++){
    double cont = histo->GetBinContent(i); 
    histo->SetBinContent(i,cont/(histo->GetBinWidth(i)));
  }
}


void MyAnalysis::RenormToRadius(TH1F* histo)
{
  for (G4int i=1; i<=histo->GetNbinsX(); i++){
    double cont = histo->GetBinContent(i); 
    double d=histo->GetBinWidth(i);
    double r0=histo->GetBinLowEdge(i);
    double r1=d+r0;
    double norm=1./(3.14159)/(r1*r1-r0*r0); 
//    G4cout << i << ": cont="<<cont << " d,r0,r1,norm" << d << "," << r0 << "," <<r1 << "," << norm << " ==== " << cont*norm << G4endl;
    histo->SetBinContent(i,cont*norm);
  }
}


